/**************************************************************************
 * FILE NAME: $RCSfile: uart_example1_mpc5500.c,v $                       *
 *                                           COPYRIGHT (c) FREESCALE 2004 *
 * DESCRIPTION:                                     All Rights Reserved   *
 * This is a sample MCF5235 program to show a simple use of the eTPU UART *
 * function.                                                              *
 * To run at the correct Baud Rate, the API timebase freq. should be set  *
 * to the system clock after prescaling.  In this example the system      *
 * clock was set to 150Mhz.  The prescaling was set to 4 in the           *
 * uart_example1.h file.                                                  *
 * Short the Transmit (eTPU0) and Receive (eTPU1) pins for this example.  *
 *                                                                        *
 *========================================================================*
 * ORIGINAL AUTHOR: Mario Perez (r2494c)                                  *
 * $Log: uart_example1_mpc5500.c,v $
 * Revision 1.2  2004/12/10 08:34:54  r47354
 * - Changes to remove etpu_config.h from builds.
 * - Formatting changes.
 *
 * Revision 1.1  2004/12/09 16:25:58  r47354
 * No changes to code, just updated for CVS and checked into CVS.
 *
 *........................................................................*
 *........................................................................*
 * 0.1   M.Perez      22/Nov/04    Initial version.                       *
 **************************************************************************/
#include "mpc5554.h"            /* mpc5554 register definitions    */
#include "mpc5554_vars.h"       /* Variables specific to mpc5554 */
#include "etpu_util.h"          /* useful utility routines         */
#include "fs_gpio.h"            /* GPIO API for the MPC5500 family */
#include "etpu_set1.h"          /* eTPU standard function set 1    */
#include "etpu_uart.h"          /* eTPU UART API                   */
#include <stdio.h>

/* User written include files */
#include "uart_example1_mpc5500.h" /*include application specific defines.*/

uint32_t *fs_free_param;

main ()
{
    int32_t error_code;
    char test_message[] = "This is an eTPU UART Ex. for the MPC5500 target";
    char store_receive_data[200];       /* place to store receive data  */
    uint8_t store_error_code[200];      /* place to store receive error codes */
    int32_t i;                  /* index variable */
    int32_t rx_data;
    vuint16_t *upper_UART_TX_SCR16;
    vuint32_t *upper_UART_TX_SCR32;
    vuint16_t *upper_UART_RX_SCR16;
    vuint32_t *upper_UART_RX_SCR32;

    upper_UART_TX_SCR32 = &ETPU.CHAN[TX_CHANNEL].SCR.R;
    upper_UART_TX_SCR16 = (vuint16_t *) upper_UART_TX_SCR32;
    upper_UART_RX_SCR32 = &ETPU.CHAN[RX_CHANNEL].SCR.R;
    upper_UART_RX_SCR16 = (vuint16_t *) upper_UART_RX_SCR32;

    FMPLL.SYNCR.R = 0x06000000; /* MFD = 0b110, RFD=0b000, 8MHz*16=128MHz */

    /* initialize eTPU hardware */
    fs_etpu_init (my_etpu_config, (uint32_t *) etpu_code, sizeof (etpu_code),
                  (uint32_t *) etpu_globals, sizeof (etpu_globals));

    /* enable all timebases */
    fs_timer_start ();

/* configure the pins on the MPC5554*/
    /* eTPU UART TX - Output */
    fs_gpio_config (FS_GPIO_ETPUA0,
                    FS_GPIO_PRIMARY_FUNCTION +
                    FS_GPIO_DRIVE_STRENGTH_50PF +
                    FS_GPIO_MAXIMUM_SLEW_RATE +
                    FS_GPIO_OUTPUT_DRAIN_DISABLE +
                    FS_GPIO_READBACK_ENABLE + 
                    FS_GPIO_OUTPUT_BUFFER_ENABLE);

    /* eTPU UART RX - Input */
    fs_gpio_config (FS_GPIO_ETPUA1,
                    FS_GPIO_PRIMARY_FUNCTION +
                    FS_GPIO_INPUT_BUFFER_ENABLE + 
                    FS_GPIO_WEAK_PULL_UP);

    *upper_UART_TX_SCR16 = 0xC0C0;  /* clear DMA/interrupt service request */
    *upper_UART_RX_SCR16 = 0xC0C0;  /* clear DMA/interrupt service request */

/*initialise the channel for UART */
    error_code = 
       fs_etpu_uart_init (TX_CHANNEL, /* tx_channel: engine A channel 0  */
                          RX_CHANNEL, /* rx_channel: engine A channel 1  */
                          FS_ETPU_PRIORITY_HIGH, /* priority: High  */
                          300,        /* baud_rate: */
                          8,  /* bits_per_data_word: */
                          FS_ETPU_UART_NO_PARITY, /* parity: */
                          16000000);  /* timebase_freq = sysclk/2/prescalers */
    /*               = 128MHz/2/4            */
    /*               = 16MHz                 */
    if (error_code != 0)
        return (TX_CHANNEL + 1);

    while (ETPU.CHAN[TX_CHANNEL].SCR.B.CIS == 0)
      {
          /* wait for init */
      }

    *upper_UART_TX_SCR16 = 0xC0C0;
    /* clear DMA/interrupt service request */

    for (i = 0; test_message[i] != NULL; i++)
      {
          fs_etpu_uart_write_transmit_data (TX_CHANNEL, 
                                            /* engine: A; channel: 0 */
                                            test_message[i]);   
                                            /* transmit_data: */

          while (ETPU.CHAN[TX_CHANNEL].SCR.B.CIS == 0)
            {
                /* wait for data to transmit */
            }

          *upper_UART_TX_SCR16 = 0xC0C0;  
          /* clear DMA/interrupt service request */

          while (ETPU.CHAN[RX_CHANNEL].SCR.B.CIS == 0)
            {
                /* wait for data to be stored in shift reg. */
            }

          rx_data = fs_etpu_uart_read_receive_data (RX_CHANNEL, 
                                                    /* rx_channel: engine A channel 1 */
                                                    &store_error_code[i]);

          store_receive_data[i] = (char) rx_data;

          *upper_UART_RX_SCR16 = 0xC0C0;        
          /* clear DMA/interrupt service request */

      }

    while (1)
      {  /* hang after message transmitted  */
      }
}

/*********************************************************************
 *
 * Copyright:
 *  Freescale Semiconductor, INC. All Rights Reserved.
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Freescale
 *  Semiconductor, Inc. This software is provided on an "AS IS"
 *  basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, Freescale
 *  Semiconductor DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED,
 *  INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 *  PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH
 *  REGARD TO THE SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF)
 *  AND ANY ACCOMPANYING WRITTEN MATERIALS.
 *
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL Freescale Semiconductor BE LIABLE FOR ANY DAMAGES WHATSOEVER
 *  (INCLUDING WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 *  BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER
 *  PECUNIARY LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.
 *
 *  Freescale Semiconductor assumes no responsibility for the
 *  maintenance and support of this software
 ********************************************************************/
